
#include "vuc-glade-plugin.h"

#define VUC_GLADE_EDIT_FONT	"monospace"
#define VUC_GLADE_CFG_LANG	"vuc-interface-lang"
#define VUC_GLADE_CFG_HEADER	"vuc-interface-header"


gchar * vuc_glade_project_langs[]= {
  "c",
//  "java",
//  "perl",
//  "python",
//  "ruby",
//  "lua",
  NULL
};


void vuc_gtk_text_buffer_set_prop (GtkTextBuffer * buffer, gchar * name, gchar * value);
VucGladeProjectPlugin * vuc_glade_project_get_data (GladeProject * project);
GtkWidget * glade_project_prefs_box_add_block (GtkWidget * main_box, gchar * label_text);
void vuc_glade_project_prefs_combo_changed (GtkWidget * widget, gpointer udata);
void vuc_glade_project_prefs_box_plugin_hook (GladeProject * project, GtkWidget * main_box,
  GtkSizeGroup * sizegroup1, GtkSizeGroup * sizegroup2, GtkSizeGroup * sizegroup3);
void vuc_glade_project_read_plugin_hook (GladeProject * project, GladeXmlNode * root);
void vuc_glade_project_write_plugin_hook (GladeProject * project, GladeXmlContext * context, GladeXmlNode * root);


gchar * vuc_gtk_text_buffer_get_text (GtkTextBuffer * buffer)
{
  GtkTextIter start, end;

  gtk_text_buffer_get_bounds(buffer, &start, &end);
  return gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
}

void vuc_gtk_text_buffer_set_prop (GtkTextBuffer * buffer, gchar * name, gchar * value)
{
  GtkTextIter start, end;
  GtkTextTag * tag= gtk_text_buffer_create_tag(buffer, NULL, name, value, NULL);

  gtk_text_buffer_get_bounds(buffer, &start, &end);
  gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
}

VucGladeProjectPlugin * vuc_glade_project_get_data (GladeProject * project)
{
  if (!project->plugin) {
    GtkTextBuffer * txt= gtk_text_buffer_new(NULL);
    vuc_gtk_text_buffer_set_prop(txt, "font", VUC_GLADE_EDIT_FONT);

    project->plugin= g_malloc0(sizeof(VucGladeProjectPlugin));
    ((VucGladeProjectPlugin *)project->plugin)->lang= -1;
    ((VucGladeProjectPlugin *)project->plugin)->header= txt;
  }
  return project->plugin;
}

static void
glade_project_write_to_comment (GladeXmlContext *context,
				GladeXmlNode    *root,
				gchar           *name,
				gchar           *value)
{
	GladeXmlNode *node;
	gchar *comment = g_strdup_printf (" %s %s ", name, value ? value : "");

	node = glade_xml_node_new_comment (context, comment);
	glade_xml_node_append_child (root, node);
	g_free (comment);
}

static gchar *
glade_project_read_from_comment (GladeXmlNode *comment,
				 gchar        *name,
				 gboolean     string)
{
	gchar *cmt, *fmt, *buffer, *ret= NULL;
	gint name_len, cmt_len, buf_len;

	if (!glade_xml_node_is_comment (comment))
		return ret;

	cmt = glade_xml_get_content (comment);
	if (cmt)
	{
		cmt_len= strlen (cmt);
		name_len= 1+strlen (name)+1;
		if (cmt_len>name_len)
		{
			buf_len= cmt_len-name_len-1; // ignore the last char (=blank)
			buffer= g_malloc0 (buf_len+1);
			fmt= g_alloca (name_len+3+10+1);
			snprintf (fmt, name_len+3+10, string ? " %s %%%dc" : " %s %%s", name, buf_len);
			if (sscanf (cmt, fmt, buffer) == 1)
			{
				g_free (cmt);
				return buffer;
			}
			g_free (buffer);
		}
		g_free (cmt);
	}

	return ret;
}

static gchar *
glade_project_find_config_item (GladeProject *project,
				GladeXmlNode *root_node,
				gchar        *name,
				gboolean     string)
{
	gchar *ret= NULL;
	GladeXmlNode *node;

	for (node = glade_xml_node_get_children_with_comments (root_node);
	     node;
	     node = glade_xml_node_next_with_comments (node))
	{
		if ((ret= glade_project_read_from_comment (node, name, string)))
			break;
	}

	return ret;
}

GtkWidget *
glade_project_prefs_box_add_block (GtkWidget * main_box, gchar * label_text)
{
	gchar * string = g_strdup_printf ("<b>%s</b>", label_text);
	GtkWidget * frame = gtk_frame_new (NULL);
	GtkWidget * alignment = gtk_alignment_new (0.5F, 0.5F, 0.95F, 1.00F);
	GtkWidget * label;

	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 5, 0, 5, 0);

	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);

	label = gtk_label_new (string);
	g_free (string);
	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);

	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
	gtk_container_add (GTK_CONTAINER (frame), alignment);

	gtk_box_pack_start (GTK_BOX (main_box), frame, TRUE, TRUE, 2);
	return alignment;
}

void vuc_glade_project_prefs_combo_changed (GtkWidget * widget, gpointer udata)
{
  GladeProject * project= udata;
  VucGladeProjectPlugin * data= vuc_glade_project_get_data(project);

  data->lang= gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void vuc_glade_project_prefs_box_plugin_hook (
  GladeProject * project, GtkWidget * main_box,
  GtkSizeGroup * sizegroup1, GtkSizeGroup * sizegroup2, GtkSizeGroup * sizegroup3)
{
  VucGladeProjectPlugin * data= vuc_glade_project_get_data(project);
  GtkWidget * container, * table, * label, * scr, * edit;
  gint i;

  GList * ch= gtk_container_get_children(GTK_CONTAINER(main_box));
  for ( ; ch; ch= ch->next)
    gtk_box_set_child_packing(GTK_BOX(main_box), ch->data, FALSE, FALSE, 4, GTK_PACK_START);

  container= glade_project_prefs_box_add_block (main_box, _("VUC options"));
  table= gtk_table_new(2, 2, FALSE);
  gtk_container_add(GTK_CONTAINER(container), table);

  label= gtk_label_new(_("Programming language"));
  gtk_table_attach(GTK_TABLE(table), label, 0,1,  0,1, GTK_SHRINK, GTK_SHRINK, 2, 4);
  data->lang_combo= gtk_combo_box_new_text();
  for (i= 0; vuc_glade_project_langs[i]; i++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(data->lang_combo), vuc_glade_project_langs[i]);
  gtk_table_attach(GTK_TABLE(table), data->lang_combo, 1,2,  0,1, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 2, 4);
  gtk_signal_connect(GTK_OBJECT(data->lang_combo), "changed", G_CALLBACK(vuc_glade_project_prefs_combo_changed), project);

  label= gtk_label_new(_("Header"));
  gtk_table_attach(GTK_TABLE(table), label, 0,1,  1,2, GTK_SHRINK, GTK_EXPAND|GTK_FILL, 2, 4);
  edit= gtk_text_view_new_with_buffer(data->header);
  scr= gtk_scrolled_window_new(NULL, NULL);
  gtk_container_add(GTK_CONTAINER(scr), edit);
  gtk_widget_set_size_request(scr, 400, 300);
  gtk_table_attach(GTK_TABLE(table), scr, 1,2,  1,2, GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 2, 4);
}

void vuc_glade_project_read_plugin_hook (GladeProject * project, GladeXmlNode * root)
{
  VucGladeProjectPlugin * data= vuc_glade_project_get_data(project);
  GtkTextIter start, end;
  gchar * str;
  gint i;

  data->lang= -1;
  str= glade_project_find_config_item(project, root, VUC_GLADE_CFG_LANG, FALSE);
  if (str)
    for (i= 0; vuc_glade_project_langs[i]; i++)
      if (strcmp(str, vuc_glade_project_langs[i])==0) {
        data->lang= i;
        gtk_combo_box_set_active(GTK_COMBO_BOX(data->lang_combo), i);
        break;
      }

  gtk_text_buffer_get_bounds(data->header, &start, &end);
  gtk_text_buffer_delete(data->header, &start, &end);
  str= glade_project_find_config_item(project, root, VUC_GLADE_CFG_HEADER, TRUE);
  gtk_text_buffer_insert_at_cursor(data->header, str, -1);
  vuc_gtk_text_buffer_set_prop(data->header, "font", VUC_GLADE_EDIT_FONT);
}

void vuc_glade_project_write_plugin_hook
  (GladeProject * project, GladeXmlContext * context, GladeXmlNode * root)
{
  VucGladeProjectPlugin * data= vuc_glade_project_get_data(project);
  GtkTextIter start, end;
  gchar * hdr;

  if (data->lang>=0)
    glade_project_write_to_comment(context, root, VUC_GLADE_CFG_LANG, vuc_glade_project_langs[data->lang]);
  gtk_text_buffer_get_start_iter(data->header, &start);
  gtk_text_buffer_get_end_iter(data->header, &end);
  hdr= gtk_text_buffer_get_text(data->header, &start, &end, TRUE);
  glade_project_write_to_comment(context, root, VUC_GLADE_CFG_HEADER, hdr);
}
